#include <msp430.h> // includes msp430fr5989.h

#define XT1_PORT_DIR	PJDIR
#define XT1_PORT_SEL0	PJSEL0
#define XT1_IN_MASK     BIT4
#define XT1_OUT_MASK    BIT5

#define LED1_OUT		P4OUT
#define LED2_OUT		P4OUT
#define LED1			BIT2
#define LED2			BIT1

// These two ISRs should catch any Timer A0 interrupt.
#pragma vector=TIMER0_A0_VECTOR
__interrupt void TIMER0_A0_ISR(void)
{						// TA0 CCR0 interrupt
    	LED1_OUT ^= LED1;
    	_BIC_SR_IRQ(LPM3_bits);
}

#pragma vector=TIMER0_A1_VECTOR

__interrupt void TIMER0_A1_ISR(void)
{
  switch(__even_in_range(TA0IV,TA0IV_TAIFG))
  {
    case 0x00:	break;	// no interrupt pending
    case 0x02:	break;	// TA0 CCR1 interrupt
    case 0x04:	break;	// TA0 CCR2 interrupt
    case 0x06:	break;	// TA0 CCR3 interrupt
    case 0x08:	break;	// TA0 CCR4 interrupt
    case 0x0A:	break;	// TA0 CCR5 interrupt
    case 0x0C:	break;	// TA0 CCR6 interrupt
    case 0x0E:			// TA0 overflow interrupt
    {
    	LED2_OUT ^= LED2;
    	_BIC_SR_IRQ(LPM3_bits);
    	break;
    }
    default:			break;
  }
}


int main(void) {

	WDTCTL = WDTPW | WDTHOLD;	// Stop watchdog timer

	XT1_PORT_DIR |= ( XT1_IN_MASK | XT1_OUT_MASK );	// enable XTAL1
	XT1_PORT_SEL0 |= ( XT1_IN_MASK | XT1_OUT_MASK );
	P4DIR |= ( LED1 | LED2 );	// enable LEDs
	PM5CTL0 &= ~LOCKLPM5;

	TA0IV = 0x00;
	TA0IV = 0x00;

// ** Clock: *********************************************************************************************************************
	CSCTL0 = CSKEY;
	CSCTL1 = 0x0048;				// F_DCO = 16MHz
	CSCTL2 = 0x0033;				// SMCLK_SRC = DCO; MCLK_SRC = DCO; ACLK = LFXT
	CSCTL3 = 0x0001;				// Divide Sources for ACLK by 1 , for SMCLK by 1 und fr MCLK durch 2 teilen.
									// => ACLK = 32,768kHz , MCLK = 8MHz , SMCLK = 16MHz
	CSCTL0_H = 0;					// lock CSCTL

// ** Timer: *********************************************************************************************************************
#define USE_ACLK

#ifdef USE_ACLK
	TA0CCR0 = 10000;
	TA0CTL = 0x01C0;	// Use ACLK as timer source, divide by 8
						// TA0CTL: SMCLK -> |= 0x0200; ACLK -> |= 0x0100;  Divider :  1 -> |= 0x000; 2 -> |= 0x0040; 4 -> |= 0x0080; 8 -> |= 0x00C0
#endif
#ifdef USE_SMCLK
	TA0CCR0 = 000;
	TA0CTL = 0x02C0;	// Use SMCLK as Timer Source, divide by 8
#endif
	TA0CCTL0 = CCIE;	// Enable Interrupt when timer counts to TA0CCR0.
// *******************************************************************************************************************************


// *******************************************************************************************************************************
// **** MAIN LOOP ****************************************************************************************************************
	while(1)
	{
        TA0CTL |= (TACLR + MC_1 + TAIE);	// Reset and start the Timer;
#ifdef USE_ACLK
        __bis_SR_register(LPM0_bits+GIE);	// This could be LPM3, but everything is supposed to be exactly the same, to allow for comparison.
#endif
#ifdef USE_SMCLK
        __bis_SR_register(LPM0_bits+GIE);
#endif
	}
// *** END OF MAIN LOOP **********************************************************************************************************
// *******************************************************************************************************************************

	return 0;
}

/* 	What should happen?
 * 		The timer should count up to the value stored in TA0CCR0 and then set the TA0CCR0 CCIFG interrupt flag. On the next cycle of the TA0 clock source TA0R should be reset, and TA0IFG (overflow)
 * 		should be set. So basically two interrupts with one clock delay between them -> both LEDs should blink. Given the SMCLK frequency of 16MHz, the timer predivider of 8, and the TA0CCR0 value
 * 		of 10000 the interrupt frequency is expected to be 200Hz. With a ACLK frequency of 32768kHz the interrupt frequency should be about 0.41Hz .
 *
 * 		With a timer frequency of 2MHz one cycle should be 0.5s. According to datasheet LPM0 wakeup-time is 0.4s + (1.5/f_DCO)
 * 		which gives in this case a wakeup-time of 0.5s. As this wakeup-time is specified for a operating voltage of at leas 2.2V the actual value might be greater as I am using 1.8V.
 * 		Therefore the delay between the two toggles should be at least 1s, possibly more.
 *
 * 	What does happen?
 * 		When using SMCLK both LEDs toggle with about 3s delay between the two different LEDs. The toggle period is 5ms for both LEDs, which means the interrupts occur with a frequency of 200Hz.
 * 		This is just as expected.
 *
 * 		When using ACLK LED1 toggles with a frequecy below 1Hz as expected, LED2 does nothing.
 *
 */

